# This file makes the unittests directory a package in order to allow the contained test modules to import each other.
# Some IDEs handle this automatically, however a pure `$ pytest unittest` call requires this file.
#
# See also https://docs.pytest.org/en/6.2.x/goodpractices.html
